@@ -327,6 +327,8 @@ int ipkg_multiple_files_scan(ipkg_conf_t *conf, int argc, char **argv)
 	  //if (strcmp (tmp + (tmplen - strlen (DPKG_PKG_EXTENSION)), DPKG_PKG_EXTENSION) != 0)
 	  //     continue;
 	
+          ipkg_message(conf, IPKG_DEBUG2, "Debug mfs: %s  \n",filename );
+
 	  err = ipkg_prepare_url_for_install(conf, filename, &argv[i]);
 	  if (err)
 	    return err;
@@ -395,6 +397,7 @@ static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char **argv)
      for (i=0; i < argc; i++) {
 	  arg = argv[i];
 
+          ipkg_message(conf, IPKG_DEBUG2, "Debug install_cmd: %s  \n",arg );
           err = ipkg_prepare_url_for_install(conf, arg, &argv[i]);
           if (err != EINVAL && err != 0)
               return err;
@@ -800,7 +803,7 @@ static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	            ipkg_message(conf, IPKG_ERROR, "Package seems to be %s not installed (STATUS = NOT_INSTALLED).\n", pkg->name);
                     continue;
                }
-               ipkg_remove_pkg(conf, pkg_to_remove);
+               ipkg_remove_pkg(conf, pkg_to_remove,0);
                done = 1;
            }
            free (pkg_name);
@@ -840,7 +843,7 @@ static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv)
 			&& !pkg_has_installed_dependents(conf, pkg->parent, pkg, NULL)) {
 			 removed++;
 			 ipkg_message(conf, IPKG_NOTICE, "Removing non-user leaf package %s\n");
-			 ipkg_remove_pkg(conf, pkg);
+			 ipkg_remove_pkg(conf, pkg,0);
                          done = 1;
 		    }
 	       }
