@@ -52,6 +52,7 @@ static int prerm_deconfigure_conflictors_unwind(ipkg_conf_t *conf, pkg_t *pkg, p
 static int preinst_configure(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
 static int preinst_configure_unwind(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
 static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
+static int check_data_file_clashes_change(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
 static int check_data_file_clashes_unwind(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
 static int backup_modified_conffiles(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
 static int backup_modified_conffiles_unwind(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg);
@@ -129,7 +130,7 @@ int ipkg_install_from_file(ipkg_conf_t *conf, const char *filename)
      }
 
      ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_pkg \n",__FUNCTION__);
-     return ipkg_install_pkg(conf, pkg);
+     return ipkg_install_pkg(conf, pkg,0);
 }
 
 ipkg_error_t ipkg_install_by_name(ipkg_conf_t *conf, const char *pkg_name)
@@ -138,9 +139,29 @@ ipkg_error_t ipkg_install_by_name(ipkg_conf_t *conf, const char *pkg_name)
      pkg_t *old, *new;
      char *old_version, *new_version;
 
+     ipkg_message(conf, IPKG_DEBUG2, " Getting old  from pkg_hash_fetch \n" );
      old = pkg_hash_fetch_installed_by_name(&conf->pkg_hash, pkg_name);
+     if ( old ) 
+        ipkg_message(conf, IPKG_DEBUG2, " Old versions from pkg_hash_fetch %s \n",  old->version );
     
+     ipkg_message(conf, IPKG_DEBUG2, " Getting new  from pkg_hash_fetch \n" );
      new = pkg_hash_fetch_best_installation_candidate_by_name(conf, pkg_name);
+     if ( new ) 
+        ipkg_message(conf, IPKG_DEBUG2, " New versions from pkg_hash_fetch %s \n",  new->version );
+
+/* Pigi Basically here is broken the version stuff.
+   What's happening is that nothing provide the version to differents 
+   functions, so the returned struct is always the latest.
+   That's why the install by name don't work.
+*/
+     ipkg_message(conf, IPKG_DEBUG2, " Versions from pkg_hash_fetch in %s ", __FUNCTION__ );
+
+     if ( old ) 
+        ipkg_message(conf, IPKG_DEBUG2, " old %s ", old->version );
+     if ( new ) 
+        ipkg_message(conf, IPKG_DEBUG2, " new %s ", new->version );
+     ipkg_message(conf, IPKG_DEBUG2, " \n");
+
      if (new == NULL) {
 	  return IPKG_PKG_HAS_NO_CANDIDATE;
      }
@@ -158,7 +179,7 @@ ipkg_error_t ipkg_install_by_name(ipkg_conf_t *conf, const char *pkg_name)
                                                               /* reinstall, and some check could fail asking the "force-reinstall" option */
           } 
 	  ipkg_message(conf, IPKG_DEBUG, 
-		       "comparing visible versions of pkg %s:"
+		       "Comparing visible versions of pkg %s:"
 		       "\n\t%s is installed "
 		       "\n\t%s is available "
 		       "\n\t%d was comparison result\n",
@@ -190,7 +211,7 @@ ipkg_error_t ipkg_install_by_name(ipkg_conf_t *conf, const char *pkg_name)
 	anyone ever wants to make a nice libipkg. */
 
      ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_pkg \n",__FUNCTION__);
-     return ipkg_install_pkg(conf, new);
+     return ipkg_install_pkg(conf, new,0);
 }
 
 ipkg_error_t ipkg_install_multi_by_name(ipkg_conf_t *conf, const char *pkg_name)
@@ -391,7 +412,7 @@ int satisfy_dependencies_for(ipkg_conf_t *conf, pkg_t *pkg)
 	  if ((dep->state_status != SS_INSTALLED)
 	      && (dep->state_status != SS_UNPACKED)) {
                ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_pkg \n",__FUNCTION__);
-	       err = ipkg_install_pkg(conf, dep);
+	       err = ipkg_install_pkg(conf, dep,0);
 	       if (err) {
 		    pkg_vec_free(depends);
 		    return err;
@@ -624,7 +645,7 @@ int pkg_remove_installed_replacees(ipkg_conf_t *conf, pkg_vec_t *replacees)
 	  pkg_t *replacee = replacees->pkgs[i];
 	  int err;
 	  replacee->state_flag |= SF_REPLACE; /* flag it so remove won't complain */
-	  err = ipkg_remove_pkg(conf, replacee);
+	  err = ipkg_remove_pkg(conf, replacee,0);
 	  if (err)
 	       return err;
      }
@@ -640,7 +661,7 @@ int pkg_remove_installed_replacees_unwind(ipkg_conf_t *conf, pkg_vec_t *replacee
 	  pkg_t *replacee = replacees->pkgs[i];
 	  if (replacee->state_status != SS_INSTALLED) {
                ipkg_message(conf, IPKG_DEBUG2,"Function: %s calling ipkg_install_pkg \n",__FUNCTION__);
-	       err = ipkg_install_pkg(conf, replacee);
+	       err = ipkg_install_pkg(conf, replacee,0);
 	       if (err)
 		    return err;
 	  }
@@ -655,7 +676,7 @@ static void ipkg_install_pkg_sigint_handler(int sig)
 }
 
 /* compares versions of pkg and old_pkg, returns 0 if OK to proceed with installation of pkg, 1 otherwise */
-static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
+static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg, int message)
 {	  
      if (old_pkg) {
           char message_out[15];
@@ -664,10 +685,11 @@ static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *ol
 	  int cmp = pkg_compare_versions(old_pkg, pkg);
 	  int rc = 0;
 
-          strncpy (message_out,"Upgrading   ",12); 
+          memset(message_out,'\x0',15);
+          strncpy (message_out,"Upgrading ",strlen("Upgrading ")); 
           if ( (conf->force_downgrade==1) && (cmp > 0) ){     /* We've been asked to allow downgrade  and version is precedent */
              cmp = -1 ;                                       /* then we force ipkg to downgrade */ 
-             strncpy (message_out,"Downgrading ",12);                      /* We need to use a value < 0 because in the 0 case we are asking to */
+             strncpy (message_out,"Downgrading ",strlen("Downgrading "));         /* We need to use a value < 0 because in the 0 case we are asking to */
                                                               /* reinstall, and some check could fail asking the "force-reinstall" option */
           } 
 
@@ -678,7 +700,7 @@ static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *ol
 	       rc = 1;
 	  } else if (cmp < 0) {
 	       ipkg_message(conf, IPKG_NOTICE,
-			    "%s %s on %s from %s to %s...\n",
+			    "%s%s on %s from %s to %s...\n",
 			    message_out, pkg->name, old_pkg->dest->name, old_version, new_version);
 	       pkg->dest = old_pkg->dest;
 	       rc = 0;
@@ -700,9 +722,16 @@ static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *ol
 	  free(new_version);
 	  return rc;
      } else {
+      char message_out[15] ;
+      memset(message_out,'\x0',15);
+      if ( message ) 
+          strncpy( message_out,"Upgrading ",strlen("Upgrading ") );
+      else
+          strncpy( message_out,"Installing ",strlen("Installing ") );
 	  char *version = pkg_version_str_alloc(pkg);
+      
 	  ipkg_message(conf, IPKG_NOTICE,
-		       "Installing %s (%s) to %s...\n",
+		       "%s%s (%s) to %s...\n", message_out,
 		       pkg->name, version, pkg->dest->name);
 	  free(version);
 	  return 0;
@@ -710,13 +739,17 @@ static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *ol
 }
 
 /* and now the meat... */
-int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
+int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg, int from_upgrade)
 {
      int err = 0;
+     int message = 0;
      pkg_t *old_pkg = NULL;
      pkg_vec_t *replacees;
      abstract_pkg_t *ab_pkg = NULL;
      int old_state_flag;
+    
+     if ( from_upgrade ) 
+        message = 1;            /* Coming from an upgrade, and should change the output message */
 
      if (!pkg) {
 	  ipkg_message(conf, IPKG_ERROR,
@@ -747,11 +780,20 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 
      old_pkg = pkg_hash_fetch_installed_by_name(&conf->pkg_hash, pkg->name);
 
-     err = ipkg_install_check_downgrade(conf, pkg, old_pkg);
+     err = ipkg_install_check_downgrade(conf, pkg, old_pkg, message);
      if (err) { return err; }
 
      pkg->state_want = SW_INSTALL;
-     if (old_pkg) old_pkg->state_want = SW_DEINSTALL; /* needed for check_data_file_clashes of dependences */
+     if (old_pkg){                           /* We're up/down-grading. Let's remove the old package before installing the new one,
+                                                as we do in ipkg_upgrade  */
+         old_pkg->state_want = SW_DEINSTALL; /* needed for check_data_file_clashes of dependences */
+         old_pkg->state_flag |= SF_REPLACE;
+         err = ipkg_remove_pkg(conf, old_pkg,1);
+         if (err==IPKG_PKG_IS_ESSENTIAL) {
+               return err;
+         }
+     }
+
 
      /* Abhaya: conflicts check */
      err = check_conflicts_for(conf, pkg);
@@ -781,6 +823,8 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
      }
 
      /* We should update the filelist here, so that upgrades of packages that split will not fail. -Jamey 27-MAR-03 */
+/* Pigi: check if it will pass from here when replacing. It seems to fail */
+/* That's rather strange that files don't change owner. Investigate !!!!!!*/
      err = update_file_ownership(conf, pkg, old_pkg);
      if (err) { return err; }
 
@@ -859,6 +903,9 @@ int ipkg_install_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 		       "  installing data files\n");
 	  install_data_files(conf, pkg);
 
+/* read comments from function for detail but I will execute this here as all other tests are ok.*/
+	  err = check_data_file_clashes_change(conf, pkg, old_pkg);
+
 	  ipkg_message(conf, IPKG_INFO,
 		       "  resolving conf files\n");
 	  resolve_conffiles(conf, pkg);
@@ -1086,6 +1133,7 @@ static int backup_modified_conffiles_unwind(ipkg_conf_t *conf, pkg_t *pkg, pkg_t
      return 0;
 }
 
+
 static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
 {
      /* DPKG_INCOMPATIBILITY:
@@ -1138,8 +1186,8 @@ static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
 
 	       /* Pre-existing files are OK if owned by a package replaced by new pkg. */
 	       if (owner) {
+                    ipkg_message(conf, IPKG_DEBUG2, "Checking for replaces for %s in package %s\n", filename, owner->name);
 		    if (pkg_replaces(pkg, owner)) {
-			 ipkg_message(conf, IPKG_INFO, "Replacing pre-existing file %s owned by package %s\n", filename, owner->name);
 			 continue;
 		    }
 /* If the file that would be installed is owned by the same package, ( as per a reinstall or similar )
@@ -1178,6 +1226,57 @@ static int check_data_file_clashes(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg
      return clashes;
 }
 
+static int check_data_file_clashes_change(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
+{
+    /* Basically that's the worst hack I could do to be able to change ownership of
+       file list, but, being that we have no way to unwind the mods, due to structure
+       of hash table, probably is the quickest hack too, whishing it would not slow-up thing too much.
+       What we do here is change the ownership of file in hash if a replace ( or similar events
+       happens )
+       Only the action that are needed to change name should be considered.
+       @@@ To change after 1.0 release.
+     */
+     str_list_t *files_list;
+     str_list_elt_t *iter;
+
+     int clashes = 0;
+
+     files_list = pkg_get_installed_files(pkg);
+     for (iter = files_list->head; iter; iter = iter->next) {
+	  char *root_filename;
+	  char *filename = iter->data;
+	  root_filename = root_filename_alloc(conf, filename);
+	  if (file_exists(root_filename) && (! file_is_dir(root_filename))) {
+	       pkg_t *owner;
+	       pkg_t *obs;
+
+	       if (conf->force_overwrite) {
+		    /* but we need to change who owns this file */
+		    file_hash_set_file_owner(conf, filename, pkg);
+		    continue;
+	       }
+
+	       owner = file_hash_get_file_owner(conf, filename);
+
+	       /* Pre-existing files are OK if owned by a package replaced by new pkg. */
+	       if (owner) {
+		    if (pkg_replaces(pkg, owner)) {
+/* It's now time to change the owner of that file. 
+   It has been "replaced" from the new "Replaces", then I need to inform lists file about that.  */
+			 ipkg_message(conf, IPKG_INFO, "Replacing pre-existing file %s owned by package %s\n", filename, owner->name);
+		         file_hash_set_file_owner(conf, filename, pkg);
+			 continue;
+		    }
+	       }
+
+	  }
+	  free(root_filename);
+     }
+     pkg_free_installed_files(pkg);
+
+     return clashes;
+}
+
 static int check_data_file_clashes_unwind(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
 {
      /* Nothing to do since check_data_file_clashes doesn't change state */
@@ -1582,7 +1681,7 @@ int ipkg_remove_packages(ipkg_conf_t *conf, pkg_vec_t *pkgs_to_remove)
      /* first, remove the packages that need removing */
      for (i = 0 ; i < pkgs_to_remove->len; i++ ) {
 	  pkg_t *pkg = pkgs_to_remove->pkgs[i];
-	  err = ipkg_remove_pkg(conf, pkg);
+	  err = ipkg_remove_pkg(conf, pkg,0);
 	  if (err) return err;
      }
      return 0;
