@@ -29,7 +29,6 @@
 #include "ipkg_cmd.h"
 
 static int remove_data_files_and_list(ipkg_conf_t *conf, pkg_t *pkg);
-static int remove_conffiles(ipkg_conf_t *conf, pkg_t *pkg);
 static int remove_maintainer_scripts_except_postrm(ipkg_conf_t *conf, pkg_t *pkg);
 static int remove_postrm(ipkg_conf_t *conf, pkg_t *pkg);
 
@@ -43,16 +42,18 @@ int pkg_has_installed_dependents(ipkg_conf_t *conf, abstract_pkg_t *parent_apkg,
      abstract_pkg_t **provides = pkg->provides;
      int n_installed_dependents = 0;
      int i;
-     for (i = 0; i < nprovides; i++) {
+     for (i = 0; i <= nprovides; i++) {
 	  abstract_pkg_t *providee = provides[i];
 	  abstract_pkg_t **dependers = providee->depended_upon_by;
 	  abstract_pkg_t *dep_ab_pkg;
 	  if (dependers == NULL)
 	       continue;
 	  while ((dep_ab_pkg = *dependers++) != NULL) {
-	       if (dep_ab_pkg->state_status == SS_INSTALLED)
+	       if (dep_ab_pkg->state_status == SS_INSTALLED){
 		    n_installed_dependents++;
+               }
 	  }
+
      }
      /* if caller requested the set of installed dependents */
      if (pdependents) {
@@ -65,7 +66,7 @@ int pkg_has_installed_dependents(ipkg_conf_t *conf, abstract_pkg_t *parent_apkg,
           }
 
 	  *pdependents = dependents;
-	  for (i = 0; i < nprovides; i++) {
+	  for (i = 0; i <= nprovides; i++) {
 	       abstract_pkg_t *providee = provides[i];
 	       abstract_pkg_t **dependers = providee->depended_upon_by;
 	       abstract_pkg_t *dep_ab_pkg;
@@ -91,6 +92,7 @@ int pkg_has_installed_dependents(ipkg_conf_t *conf, abstract_pkg_t *parent_apkg,
 int ipkg_remove_dependent_pkgs (ipkg_conf_t *conf, pkg_t *pkg, abstract_pkg_t **dependents)
 {
     int i;
+    int a;
     int count;
     pkg_vec_t *dependent_pkgs = pkg_vec_alloc();
     abstract_pkg_t * ab_pkg;
@@ -115,12 +117,13 @@ int ipkg_remove_dependent_pkgs (ipkg_conf_t *conf, pkg_t *pkg, abstract_pkg_t **
     while (dependents [i] != NULL) {
         abstract_pkg_t *dep_ab_pkg = dependents[i];
 	
-	if (dep_ab_pkg->dependencies_checked == 2)
+	if (dep_ab_pkg->dependencies_checked == 2){
+	    i++;
 	    continue;	
+        }
         if (dep_ab_pkg->state_status == SS_INSTALLED) {
-            int i;
-            for (i = 0; i < dep_ab_pkg->pkgs->len; i++) {
-                pkg_t *dep_pkg = dep_ab_pkg->pkgs->pkgs[i];
+            for (a = 0; a < dep_ab_pkg->pkgs->len; a++) {
+                pkg_t *dep_pkg = dep_ab_pkg->pkgs->pkgs[a];
                 if (dep_pkg->state_status == SS_INSTALLED) {
                     pkg_vec_insert(dependent_pkgs, dep_pkg);
                     count++;
@@ -137,7 +140,7 @@ int ipkg_remove_dependent_pkgs (ipkg_conf_t *conf, pkg_t *pkg, abstract_pkg_t **
     
     
     for (i = 0; i < dependent_pkgs->len; i++) {
-        int err = ipkg_remove_pkg(conf, dependent_pkgs->pkgs[i]);
+        int err = ipkg_remove_pkg(conf, dependent_pkgs->pkgs[i],0);
         if (err)
             return err;
     }
@@ -162,12 +165,16 @@ static int user_prefers_removing_dependents(ipkg_conf_t *conf, abstract_pkg_t *a
     return 0;
 }
 
-int ipkg_remove_pkg(ipkg_conf_t *conf, pkg_t *pkg)
+int ipkg_remove_pkg(ipkg_conf_t *conf, pkg_t *pkg,int message)
 {
+/* Actually, when "message == 1" I have been called from an upgrade, and not from a normal remove
+   thus I wan't check for essential, as I'm upgrading.
+   I hope it won't break anything :) 
+*/
      int err;
      abstract_pkg_t *parent_pkg = NULL;
 	
-     if (pkg->essential) {
+     if (pkg->essential && !message) {
 	  if (conf->force_removal_of_essential_packages) {
 	       fprintf(stderr, "WARNING: Removing essential package %s under your coercion.\n"
 		       "\tIf your system breaks, you get to keep both pieces\n",
@@ -213,9 +220,10 @@ int ipkg_remove_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 	  }
      }
 
-     printf("Removing package %s from %s...\n", pkg->name, pkg->dest->name);
-     fflush(stdout);
-
+     if ( message==0 ){
+         printf("Removing package %s from %s...\n", pkg->name, pkg->dest->name);
+         fflush(stdout);
+     }
      pkg->state_flag |= SF_FILELIST_CHANGED;
 
      pkg->state_want = SW_DEINSTALL;
@@ -245,14 +253,12 @@ int ipkg_remove_pkg(ipkg_conf_t *conf, pkg_t *pkg)
      if (parent_pkg) 
 	  parent_pkg->state_status = SS_NOT_INSTALLED;
 
-     printf("Done.\n");
-
      return 0;
 }
 
 int ipkg_purge_pkg(ipkg_conf_t *conf, pkg_t *pkg)
 {
-    ipkg_remove_pkg(conf, pkg);
+    ipkg_remove_pkg(conf, pkg,0);
     return 0;
 }
 
@@ -358,6 +364,7 @@ static int remove_maintainer_scripts_except_postrm(ipkg_conf_t *conf, pkg_t *pkg
 	if (str_ends_with(globbuf.gl_pathv[i], ".postrm")) {
 	    continue;
 	}
+        ipkg_message(conf, IPKG_INFO, "  deleting %s\n", globbuf.gl_pathv[i]);
 	unlink(globbuf.gl_pathv[i]);
     }
     globfree(&globbuf);
